namespace XMLChapterCS {
	using System;
	using System.Xml;
	using System.Xml.Schema;
	using System.IO;
	using System.Text;
	using System.Net;
	using System.Web;

	/// <summary>
	///		The Validator class encapsulates various XML validation functionality
	/// </summary>
	public class XmlValidator {
		bool _valid;
		bool _logError;
		string _logFile;
		string _validationErrors = String.Empty;

		XmlTextReader xmlReader = null;
		XmlValidatingReader vReader = null;
		public XmlValidationStatus Validate(object xml,XmlSchemaCollection schemaCol, string[] dtdInfo,
			bool logError,string logFile) {
			_logError = logError;
			_logFile = logFile;
			_valid = true;

			try {
				//Check what type of XML data source was passed
				if (xml is StringReader) {
					xmlReader = new XmlTextReader((StringReader)xml);
				} else if (xml is String) {
					xmlReader = new XmlTextReader((String)xml);
				} else if (xml is Stream) {
					xmlReader = new XmlTextReader((Stream)xml);
				} else {
					throw new ApplicationException("Invalid XML data source passed.");
				}
				
				if (dtdInfo != null && dtdInfo.Length > 0) {
					XmlParserContext context = 
						new XmlParserContext(null,null,dtdInfo[0],"",dtdInfo[1],"",dtdInfo[1],"",XmlSpace.Default);
					xmlReader.MoveToContent();
					vReader = new XmlValidatingReader(xmlReader.ReadOuterXml(),XmlNodeType.Element,context);
					vReader.ValidationType = ValidationType.DTD;
				} else {
					vReader = new XmlValidatingReader(xmlReader);
					vReader.ValidationType = ValidationType.Auto;
					if (schemaCol != null) {
						vReader.Schemas.Add(schemaCol);
					}
				}

				vReader.ValidationEventHandler  += 
					new ValidationEventHandler(this.ValidationCallBack);
				// Parse through XML
				while (vReader.Read()){}
			} catch  {
				_valid = false;
			} finally {  //Close our reader
				if (vReader != null) {
					vReader.Close();
				}
			}
			//Report back to calling application
			XmlValidationStatus status = new XmlValidationStatus();
			status.Status = _valid;
			status.ErrorMessages = _validationErrors;
			return status;
		}

		private void ValidationCallBack(object sender, ValidationEventArgs args)	{
			_valid = false;  //hit callback so document has a problem
			DateTime today = DateTime.Now;
			StreamWriter writer = null;
			try {			
				if (_logError) {
					writer = new StreamWriter(_logFile,true,Encoding.ASCII);
					writer.WriteLine("Validation error in XML: ");
					writer.WriteLine();
					writer.WriteLine(args.Message + " " + today.ToString());
					writer.WriteLine();
					if (xmlReader.LineNumber > 0) {
						writer.WriteLine("Line: "+ xmlReader.LineNumber + 
							" Position: " + xmlReader.LinePosition);
					}
					writer.WriteLine();
					writer.Flush();
				} else {
					_validationErrors = args.Message + " Line: " + xmlReader.LineNumber + " Column:" + xmlReader.LinePosition + "\n\n";
				}
			}
			catch {}
			finally {
				if (writer != null) {
					writer.Close();
				}
			}
		}
	} 

	public struct XmlValidationStatus {
		public bool Status;
		public string ErrorMessages;
	}
}
