Imports System
Imports System.Web
Imports System.Web.Caching
Imports System.Xml.XPath
Imports System.Configuration

Public Class ConfigReader

  Public Shared Function GetConfigValue(ByVal xpath As String) As String
    Dim doc As XPathDocument = GetConfigDocument()
    Dim nav As XPathNavigator = doc.CreateNavigator()
    nav.MoveToRoot()
    nav.MoveToFirstChild()
    Dim it As XPathNodeIterator = nav.Select(xpath)
    If Not (it Is Nothing) Then
      it.MoveNext()
      Return it.Current.Value
    Else
      Return Nothing
    End If
  End Function


  Public Shared Function GetConnectionString(ByVal server As String) As String
    Return GetConfigValue(("SqlServer/" + GetServerType(server).ToString() + "/@ConnString"))
  End Function


  Public Shared Function GetServerType(ByVal server As String) As ServerType
    Dim currServer As String = server.ToLower()
    Dim doc As XPathDocument = GetConfigDocument()
    Dim nav As XPathNavigator = doc.CreateNavigator()
    nav.MoveToRoot()
    nav.MoveToFirstChild()
    Dim xpath As String = "//Servers/Server"
    Dim it As XPathNodeIterator = nav.Select(xpath)
    While it.MoveNext()
      'Check if server is in production
      If it.Current.Value.ToLower() = currServer Then
        it.Current.MoveToParent() 'Move to ServerNames
        it.Current.MoveToParent() 'Move to server type
        Return CType([Enum].Parse(GetType(ServerType), it.Current.Name, True), ServerType)
      End If
    End While
    'Default to dev server
    Return ServerType.Dev
  End Function


  Private Shared Function GetConfigDocument() As XPathDocument
    Dim context As HttpContext = HttpContext.Current
    'Check if config is already loaded into cache
    If context.Cache.Get("ServerConfig") Is Nothing Then
      Try
        Dim configPath As String = _
          context.Server.MapPath( _
          ConfigurationSettings.AppSettings("ServerConfig"))
        Dim doc As New XPathDocument(configPath)
        'Create file dependency for cache
        Dim cd As New CacheDependency(New String() {configPath})
        'Cache XPathDocument instance
        context.Cache.Insert("ServerConfig", doc, cd)
        Return doc
      Catch
      End Try
    Else
      'Return XPathDocument already in cache
      Return CType(context.Cache.Get("ServerConfig"), XPathDocument)
    End If
  End Function
End Class
 _

Public Enum ServerType
  Prod
  Dev
End Enum
