Imports System.Xml
Imports System.Xml.Schema
Imports System.IO
Imports System.Text
Imports System.Net
Imports System.Web

Public Class XmlValidator
  Private _valid As Boolean
  Private _logError As Boolean
  Private _logFile As String
  Private _validationErrors As String = String.Empty
  Private xmlReader As XmlTextReader = Nothing
  Private vReader As XmlValidatingReader = Nothing

  Public Function Validate(ByVal xml As Object, _
    ByVal schemaCol As XmlSchemaCollection, _
    ByVal dtdInfo() As String, ByVal logError As Boolean, _
    ByVal logFile As String) As XmlValidationStatus
    _logError = logError
    _logFile = logFile
    _valid = True

    Try
      'Check what type of XML data source was passed
      If TypeOf xml Is StringReader Then
        xmlReader = New XmlTextReader(CType(xml, StringReader))
      ElseIf TypeOf xml Is String Then
        xmlReader = New XmlTextReader(CType(xml, String))
      ElseIf TypeOf xml Is Stream Then
        xmlReader = New XmlTextReader(CType(xml, Stream))
      Else
        Throw New ApplicationException("Invalid XML data " + _
         "source passed.")
      End If
      'Hookup DTD or Schemas
      If Not (dtdInfo Is Nothing) Then
        If dtdInfo.Length > 0 Then
          Dim context As New XmlParserContext(Nothing, Nothing, _
            dtdInfo(0), "", dtdInfo(1), "", dtdInfo(1), "", _
            XmlSpace.Default)
          xmlReader.MoveToContent()
          vReader = _
            New XmlValidatingReader(xmlReader.ReadOuterXml(), _
              XmlNodeType.Element, context)
          vReader.ValidationType = ValidationType.DTD
        End If
      Else
        vReader = New XmlValidatingReader(xmlReader)
        vReader.ValidationType = ValidationType.Auto
        If Not (schemaCol Is Nothing) Then
          vReader.Schemas.Add(schemaCol)
        End If
      End If
      'Associate validating reader with callback method 
      'to handle any validation errors
      AddHandler vReader.ValidationEventHandler, _
        AddressOf Me.ValidationCallBack
      ' Parse through XML document
      While vReader.Read()
      End While
    Catch
      _valid = False
    Finally 'Close validating reader
      If Not (vReader Is Nothing) Then
        vReader.Close()
      End If
    End Try

    'Report back to calling application
    Dim status As New XmlValidationStatus
    status.Status = _valid
    status.ErrorMessages = _validationErrors
    Return status
  End Function


  Private Sub ValidationCallBack(ByVal sender As Object, _
    ByVal args As ValidationEventArgs)
    _valid = False 'hit callback so document has a problem
    Dim today As DateTime = DateTime.Now
    Dim writer As StreamWriter = Nothing
    Try
      If _logError Then 'Handle logging to logfile
        writer = New StreamWriter(_logFile, True, Encoding.ASCII)
        writer.WriteLine("Validation error in XML: ")
        writer.WriteLine()
        writer.WriteLine((args.Message + " " + today.ToString()))
        writer.WriteLine()
        If xmlReader.LineNumber > 0 Then
          writer.WriteLine(("Line: " + xmlReader.LineNumber + _
          " Position: " + xmlReader.LinePosition))
        End If
        writer.WriteLine()
        writer.Flush()
      Else 'Track error messages
        _validationErrors = args.Message + " Line: " + _
        xmlReader.LineNumber.ToString() + _
        " Column:" + xmlReader.LinePosition.ToString() + _
        ControlChars.Lf + ControlChars.Lf
      End If
    Catch
    Finally 'Ensure StreamWriter gets closed
      If Not (writer Is Nothing) Then
        writer.Close()
      End If
    End Try
  End Sub

End Class

Public Structure XmlValidationStatus
  Public Status As Boolean
  Public ErrorMessages As String
End Structure
