<%@Page Language="VB" Debug="True" Trace="False"%>
<%@Import Namespace="System.Data" %>
<%@Import Namespace="System.Data.OleDb" %>

<script runat="server">

<%-------------------------------------------------------------%>

' declare page-level variables
Dim sCustSql, sOrdersSql, sDetailsSql As String
Dim sConnect, sSchemaFile As String

<%-------------------------------------------------------------%>

Function FillDataSet(bLoadSchema As Boolean) As DataSet

  ' get DataSet with rows from Northwind tables
  Dim oDataSet As New DataSet()

  If bLoadSchema = True Then
    oDataSet.ReadXmlSchema(sSchemaFile)
  End If

  Dim oConnect As New OleDbConnection(sConnect)

  Try

    ' fill DataSet with three tables
    Dim oDA As New OleDbDataAdapter(sCustSQL, oConnect)
    oDA.MissingSchemaAction = lstMissingSchema.SelectedValue
    oConnect.Open()
    oDA.Fill(oDataSet, "Customers")
    oDA.SelectCommand.CommandText = sOrdersSql
    oDA.Fill(oDataSet, "Orders")
    oDA.SelectCommand.CommandText = sDetailsSql
    oDA.Fill(oDataSet, "OrderDetails")
    oConnect.Close()

    If bLoadSchema = False Then
      ' create relations between the tables
      Dim oRel As New DataRelation("CustOrders", _
        oDataSet.Tables("Customers").Columns("CustomerID"), _
        oDataSet.Tables("Orders").Columns("CustomerID"))
      oDataSet.Relations.Add(oRel)
      oRel = New DataRelation("OrdersODetails", _
        oDataSet.Tables("Orders").Columns("OrderID"), _
        oDataSet.Tables("OrderDetails").Columns("OrderID"))
      oDataSet.Relations.Add(oRel)
    End If

    Return oDataSet

  Catch oErr As Exception

    ' be sure to close connection if error occurs
    If oConnect.State <> ConnectionState.Closed  Then
      oConnect.Close()
    End If

    ' display error message in page
    lblResult.Text &= "ERROR: " & oErr.Message & "<br />"

  End Try

End Function

<%-------------------------------------------------------------%>

Sub CreateSQLStatements()

  ' create SQL statements and get connection string
  sCustSql = "SELECT CustomerID, CompanyName, City, Country " _
    & "FROM Customers WHERE CustomerID LIKE 's%'"
  sOrdersSql = "SELECT CustomerID, OrderID, OrderDate, ShippedDate " _
    & "FROM Orders WHERE CustomerID LIKE 's%'"
  sDetailsSql = "SELECT [Order Details].OrderID, Products.ProductID, " _
    & "Products.ProductName, [Order Details].Quantity, " _
    & "[Order Details].UnitPrice " _
    & "FROM [Order Details] JOIN Products " _
    & "ON [Order Details].ProductID = Products.ProductID " _
    & "WHERE [Order Details].OrderID IN " _
    & " (SELECT OrderID FROM Orders " _
    & "  WHERE CustomerID LIKE 's%')"
  sConnect = ConfigurationSettings.AppSettings( _
             "NorthwindOleDbConnectString")
End Sub

<%-------------------------------------------------------------%>

Sub DoTest(sender As Object, e As EventArgs)

  ' declare local variables

  Dim iCount As Integer
  Dim iLoop As Integer
  Dim oDS As DataSet
  Dim dStart As DateTime
  Dim dDiff1, dDiff2 As TimeSpan

  ' check if running on local server to limit load
  ' on public samples server
  Dim sPath As String = Request.Url.ToString()
  If sPath.IndexOf("http://localhost/") = 0 Then
    iCount = 100
  Else
    iCount = 10
  End If

  ' calculate schema file path
  sSchemaFile = Request.MapPath(Request.ApplicationPath) _
              & "\dataaccess\datasetschema.xsd"

  CreateSQLStatements()

  ' load DataSet with schema
  Trace.Write("With Schema", "Start")
  dStart = DateTime.Now
  For iLoop = 1 To iCount
    oDS = FillDataSet(True)
  Next
  dDiff1 = DateTime.Now.Subtract(dStart)
  Trace.Write("With Schema", "End")
  lblResult.Text &= "Loaded DataSet with schema " _
    & iCount.ToString() & " times in " _
    & dDiff1.TotalMilliseconds.ToString() & " ms.<br />"

  ' load DataSet without schema - can't do it
  ' when MissingSchemaAction is Ignore or Error
  If lstMissingSchema.SelectedValue = 1 _
  Or lstMissingSchema.SelectedValue = 4 Then
    Trace.Write("Without Schema", "Start")
    dStart = DateTime.Now
    For iLoop = 1 To iCount
       oDS = FillDataSet(False)
    Next
    dDiff2 = DateTime.Now.Subtract(dStart)
    Trace.Write("Without Schema", "End")
    lblResult.Text &= "Loaded DataSet without schema " _
      & iCount.ToString() & " times in " _
      & dDiff2.TotalMilliseconds.ToString() & " ms.<br />"

    ' calculate difference
    Dim fRatio As Decimal = (dDiff1.TotalMilliseconds _
      - dDiff2.TotalMilliseconds) / dDiff2.TotalMilliseconds
    lblResult.Text &= "With schema is " & Math.Abs(fRatio).ToString("p")
    If dDiff1.TotalMilliseconds > dDiff2.TotalMilliseconds Then
      lblResult.Text &= " slower."
    Else
      lblResult.Text &= " faster."
    End If

  End If

End Sub

<%-------------------------------------------------------------%>

Sub ShowSchema(sender As Object, e As EventArgs)

  If lstMissingSchema.SelectedValue = 1 _
  Or lstMissingSchema.SelectedValue = 4 Then
    CreateSQLStatements()
    Dim oDS As DataSet = FillDataSet(False)
    lblSchema.Text = "<pre>" & Server.HtmlEncode(oDS.GetXmlSchema()) & "</pre>"
  Else
    lblSchema.Text = "Cannot create schema dynamically for Ignore or Error values"
  End If

End Sub

</script>

<%-------------------------------------------------------------%>

<html>
<head>
<!-- #include file="../global/style.inc" -->
<title>Filling a Dataset With and Without a Schema</title>
</head>
<body>

<span class="heading">Filling a Dataset With and Without a Schema</span><hr />

<form runat="server">

  MissingSchemaAction:
  <asp:DropDownList id="lstMissingSchema" runat="server">
    <asp:ListItem Text="Add (default)" Value="1" />
    <asp:ListItem Text="AddWithKey" Value="4" />
    <asp:ListItem Text="Ignore" Value="2" />
    <asp:ListItem Text="Error" Value="3" />
  </asp:DropDownList> &nbsp;
  <asp:Button Text="Compare" onclick="DoTest" runat="server" /> &nbsp;
  <asp:Button Text="View schema" onclick="ShowSchema" runat="server" /><p />
  <asp:Label id="lblResult" EnableViewState="False" runat="server" /><p />
  <asp:Label id="lblSchema" EnableViewState="False" runat="server" />

</form>

<span class="cite">
Remember to edit the connection strings in web.config file
for the Northwind sample database.
</span>
<hr />
<!-- #include file="../global/viewsource.inc" -->
<!-- #include file="../global/footer.inc" --><p />

</body>
</html>

