<%@Page Language="VB" EnableViewState="False" Debug="True" %>
<%@Import Namespace="System.Data" %>
<%@Import Namespace="System.Data.OleDb" %>
<%@Import Namespace="System.Drawing" %>

<%-------------------------------------------------------------%>

<script runat="server">

' variable to hold reference to DataSet across routines
Dim oDataSet As DataSet

Sub Page_Load()

  'fill the dataset with some rows from database
  FillDataSet("c%")

  ' bind the data to the grid for display
  dgr1.DataSource = oDataSet
  dgr1.DataMember = "Customers"
  dgr1.DataBind()

End Sub

<%-------------------------------------------------------------%>

Sub FillDataSet(sCustID As String)

  ' get DataSet with rows from Northwind tables

  Dim sCustSql As String _
    = "SELECT CustomerID, CompanyName, City, Country " _
    & "FROM Customers WHERE CustomerID LIKE '" & sCustID & "'"
  Dim sOrdersSql As String _
    = "SELECT CustomerID, OrderID, OrderDate, ShippedDate " _
    & "FROM Orders WHERE CustomerID LIKE '" & sCustID & "'"
  Dim sDetailsSql As String _
    = "SELECT [Order Details].OrderID, Products.ProductID, " _
    & "Products.ProductName, [Order Details].Quantity, " _
    & "[Order Details].UnitPrice " _
    & "FROM [Order Details] JOIN Products " _
    & "ON [Order Details].ProductID = Products.ProductID " _
    & "WHERE [Order Details].OrderID IN " _
    & " (SELECT OrderID FROM Orders " _
    & "  WHERE CustomerID LIKE '" & sCustID & "')"

  Dim sConnect As String _
    = ConfigurationSettings.AppSettings("NorthwindOleDbConnectString")
  Dim oConnect As New OleDbConnection(sConnect)
  oDataSet = New DataSet()

  Try

    ' fill DataSet with three tables
    Dim oDA As New OleDbDataAdapter(sCustSQL, oConnect)
    oConnect.Open()
    oDA.Fill(oDataSet, "Customers")
    oDA.SelectCommand.CommandText = sOrdersSql
    oDA.Fill(oDataSet, "Orders")
    oDA.SelectCommand.CommandText = sDetailsSql
    oDA.Fill(oDataSet, "OrderDetails")
    oConnect.Close()

    ' create relations between the tables
    Dim oRel As New DataRelation("CustOrders", _
      oDataSet.Tables("Customers").Columns("CustomerID"), _
      oDataSet.Tables("Orders").Columns("CustomerID"))
    oDataSet.Relations.Add(oRel)
    oRel = New DataRelation("OrdersODetails", _
      oDataSet.Tables("Orders").Columns("OrderID"), _
      oDataSet.Tables("OrderDetails").Columns("OrderID"))
    oDataSet.Relations.Add(oRel)

  Catch oErr As Exception

    ' be sure to close connection if error occurs
    If oConnect.State <> ConnectionState.Closed  Then
      oConnect.Close()
    End If

    ' display error message in page
    lblErr.Text = oErr.Message

  End Try

End Sub

<%-------------------------------------------------------------%>

Sub BindOrdersGrid(sender As Object, e As DataGridItemEventArgs)

  ' see what type of row (header, footer, item, etc.) caused the event
  Dim oType As ListItemType = CType(e.Item.ItemType, ListItemType)

  ' only process it if it's an Item or AlternatingItem event
  If oType = ListItemType.Item _
  Or oType = ListItemType.AlternatingItem Then

    ' get a reference to the DataGrid control in this row
    Dim oGrid As DataGrid = CType(e.Item.FindControl("dgr2"), DataGrid)

    ' get value of CustomerID for this row from DataKeys collection
    Dim sKey As String = dgr1.DataKeys(e.Item.ItemIndex)

    ' get a DataView containing just the current row in
    ' the Customers table within the DataSet
    Dim oView, oChildView As DataView
    oView = oDataSet.Tables("Customers").DefaultView
    oView.RowFilter = "CustomerID = '" & sKey & "'"
    oChildView = oView(0).CreateChildView( _
                 oDataSet.Relations("CustOrders"))

    ' find rows that have not yet shipped and delete them
    ' have to go backwards through row collection to avoid
    ' errors as indexes of rows change when one is deleted
    For iIndex As Integer = (oChildView.Count - 1) To 0 Step -1
      If oChildView(iIndex)("ShippedDate").ToString() = "" Then
        oChildView(iIndex).Delete()
      End If
    Next

    ' bind nested "orders" DataGrid to child DataView
    oGrid.DataSource = oChildView
    oGrid.DataBind()

  End If

End Sub

<%-------------------------------------------------------------%>

Sub BindOrderItemsGrid(sender As Object, e As DataGridItemEventArgs)

  ' see what type of row (header, footer, item, etc.) caused the event
  Dim oType As ListItemType = CType(e.Item.ItemType, ListItemType)

  ' only process it if it's an Item or AlternatingItem event
  If oType = ListItemType.Item _
  Or oType = ListItemType.AlternatingItem Then

    ' get the value of the CustomerID column
    ' argument sender is a reference to the containing DataGrid
    Dim iKey As Integer = sender.DataKeys(e.Item.ItemIndex)

    ' get a reference to the DataGrid control in this row
    Dim oGrid As DataGrid = CType(e.Item.FindControl("dgr3"), DataGrid)

    ' get a DataView containing just the current row in
    ' the Orders table within the DataSet
    Dim oView, oChildView As DataView
    oView = oDataSet.Tables("Orders").DefaultView
    oView.RowFilter = "OrderID = " & iKey
    oChildView = oView(0).CreateChildView( _
                 oDataSet.Relations("OrdersODetails"))

    ' find rows where unit price is greater
    ' than $10.00 and highlight product name
    For iIndex As Integer = 0 To oChildView.Count - 1
      If oChildView(iIndex)("UnitPrice") > 10 Then
        oChildView(iIndex)("ProductName") = "<b><i>" _
                  & oChildView(iIndex)("ProductName") & "</i></b>"
      End If
    Next

    ' bind nested "order details" DataGrid to child DataView
    oGrid.DataSource = oChildView
    oGrid.DataBind()

  End If

End Sub

</script>

<%-------------------------------------------------------------%>
<%-------------------------------------------------------------%>

<html>
<head>
<!-- #include file="../global/style.inc" -->
<title>Filling Nested DataGrids from a DataSet</title>
</head>
<body>

<span class="heading">Filling Nested DataGrids from a DataSet</span><hr />

<form runat="server">

  <asp:Label id="lblErr" EnableViewState="False" runat="server" /><p />

  <asp:DataGrid id="dgr1" runat="server"
       Font-Size="10" Font-Name="Tahoma,Arial,Helvetica,sans-serif"
       BorderStyle="None" BorderWidth="1px" BorderColor="#deba84"
       BackColor="#DEBA84" CellPadding="5" CellSpacing="1"
       AutoGenerateColumns="False"
       DataKeyField="CustomerID"
       OnItemDataBound="BindOrdersGrid" >
    <HeaderStyle Font-Bold="True" ForeColor="#ffffff" BackColor="#b50055" />
    <ItemStyle BackColor="#FFF7E7" VerticalAlign="Top" />
    <AlternatingItemStyle backcolor="#ffffc0" />
    <Columns>
      <asp:TemplateColumn HeaderText="Customer Details">
        <ItemTemplate>
          <b><%# Container.DataItem("CompanyName") %></b><br />
          City: <%# Container.DataItem("City") %><br />
          Country: <%# Container.DataItem("Country") %><br />
          CustomerID: "<%# Container.DataItem("CustomerID") %>"
        </ItemTemplate>
      </asp:TemplateColumn>
      <asp:TemplateColumn HeaderText="Order History">
        <ItemTemplate>

          <asp:DataGrid id="dgr2" runat="server"
               BorderStyle="None" BorderWidth="0" BackColor="#deba84"
               CellPadding="5" CellSpacing="2" Width="100%"
               AutoGenerateColumns="False"
               DataKeyField="OrderID"
               OnItemDataBound="BindOrderItemsGrid" >
            <HeaderStyle BackColor="#c0c0c0" />
            <ItemStyle Font-Bold="True" VerticalAlign="Top" />
            <Columns>
              <asp:BoundColumn DataField="OrderID" HeaderText="Number" />
              <asp:TemplateColumn HeaderText="Details">
                <ItemTemplate>
                  <asp:Label runat="server"
                       Text='<%# DataBinder.Eval(Container.DataItem, _
                             "OrderDate", "{0:dddd dd MMM yyyy}") %>' />

                  <asp:DataGrid id="dgr3" runat="server"
                       CellPadding="3" CellSpacing="0" Width="100%"
                       BorderStyle="None" BorderWidth="0"
                       AutoGenerateColumns="False" >
                    <HeaderStyle BackColor="#c0c0c0" />
                    <Columns>
                      <asp:BoundColumn DataField="ProductID"
                           HeaderText="ID" />
                      <asp:BoundColumn DataField="ProductName"
                           HeaderText="Product" />
                      <asp:BoundColumn DataField="Quantity"
                           ItemStyle-HorizontalAlign="Right"
                           HeaderStyle-HorizontalAlign="Right"
                           HeaderText="Qty" />
                      <asp:BoundColumn DataField="UnitPrice"
                           DataFormatString="${0:f2}"
                           ItemStyle-HorizontalAlign="Right"
                           HeaderStyle-HorizontalAlign="Right"
                           HeaderText="Price"/>
                    </Columns>
                  </asp:DataGrid>

                </ItemTemplate>
              </asp:TemplateColumn>
            </Columns>
          </asp:DataGrid>

        </ItemTemplate>
      </asp:TemplateColumn>
    </Columns>
  </asp:DataGrid>

</form>

<span class="cite">
Remember to edit the connection strings in your web.config
file for the Northwind sample database.
</span>
<hr />
<!-- #include file="../global/viewsource.inc" -->
<!-- #include file="../global/footer.inc" -->

</body>
</html>

