<%@Page Language="VB" Debug="True" Trace="False" %>
<%@Import Namespace="System.Drawing" %>
<%@Import Namespace="System.Data" %>
<%@Import Namespace="System.Data.OleDb" %>

<%-------------------------------------------------------------%>

<script runat="server">

' page-level variables to hold references to controls that
' are added dynamically. Allows they to be referenced in
' event handlers that run after the Page_Load event
Dim oGrid As DataGrid
Dim oLabel As Label

Sub Page_Load()

  ' create a DataGrid control
  oGrid = New DataGrid()

  ' set control properties
  With oGrid
    .id = "dgr1"
    .BorderStyle = BorderStyle.None
    .BorderWidth = Unit.Pixel(0)
    .BackColor = Color.FromName("#deba84")
    .CellPadding = 3
    .CellSpacing = 0
    .DataKeyField = "CustomerID"
    .Width = Unit.Percentage(100)
    .AutoGenerateColumns = False
    .ItemStyle.BackColor = Color.FromName("#fff7e7")
    .AlternatingItemStyle.BackColor = Color.FromName("#ffffc0")
  End With

  ' create a column for the DataGrid control
  ' and set properties
  Dim oCol1 As New EditCommandColumn()
  With oCol1
    .EditText = "Edit"
    .CancelText = "Cancel"
    .UpdateText = "Update"
  End With

  ' add column to DataGrid
  oGrid.Columns.Add(oCol1)

  ' repeat for remaining columns
  Dim oCol2 = New BoundColumn()
  With oCol2
    .DataField = "CustomerID"
    .HeaderText = "ID"
    .ReadOnly = True
  End With
  oGrid.Columns.Add(oCol2)

  Dim oCol3 = New BoundColumn()
  With oCol3
    .DataField = "CompanyName"
    .HeaderText = "Company Name"
  End With
  oGrid.Columns.Add(oCol3)

  Dim oCol4 = New BoundColumn()
  With oCol4
    .DataField = "City"
    .HeaderText = "City"
  End With
  oGrid.Columns.Add(oCol4)

  Dim oCol5 = New BoundColumn()
  With oCol5
    .DataField = "Country"
    .HeaderText = "Country"
  End With
  oGrid.Columns.Add(oCol5)

  Dim oCol6 = New BoundColumn()
  With oCol6
    .DataField = "Phone"
    .HeaderText = "Phone"
  End With
  oGrid.Columns.Add(oCol6)

  ' add event handlers to the grid
  AddHandler oGrid.EditCommand, New DataGridCommandEventHandler(AddressOf DoItemEdit)
  AddHandler oGrid.UpdateCommand, New DataGridCommandEventHandler(AddressOf DoItemUpdate)
  AddHandler oGrid.CancelCommand, New DataGridCommandEventHandler(AddressOf DoItemCancel)

  ' create new Label control and set properties
  oLabel = New Label()
  With oLabel
    .id = "lblErr"
    .EnableViewState = False
  End With

  ' add new controls to page as children of <form>
  frmMain.Controls.Add(oGrid)
  frmMain.Controls.Add(oLabel)

  ' only need to databind if it is not a postback
  ' viewstate used to populate dynamically added controls
  If Not Page.IsPostback Then
    oGrid.DataSource = GetCustomers()
    oGrid.DataBind()
    oGrid.HeaderStyle.Font.Bold = True
    oGrid.HeaderStyle.ForeColor = Color.FromName("#ffffff")
    oGrid.HeaderStyle.BackColor = Color.FromName("#b50055")
  End If

End Sub

<%-------------------------------------------------------------%>

Function GetCustomers() As OleDbDataReader

  Dim sSelect As String _
    = "SELECT CustomerID, CompanyName, City, Country, Phone " _
    & "FROM Customers WHERE CustomerID LIKE 'c%'"

  ' get DataReader for rows from Northwind Customers table
  Dim sConnect As String _
    = ConfigurationSettings.AppSettings("NorthwindOleDbConnectString")
  Dim oConnect As New OleDbConnection(sConnect)

  Try

    oConnect.Open()
    Dim oCommand As New OleDbCommand(sSelect, oConnect)
    Return oCommand.ExecuteReader(CommandBehavior.CloseConnection)

  Catch oErr As Exception

    ' be sure to close connection if error occurs
    If oConnect.State <> ConnectionState.Closed  Then
      oConnect.Close()
    End If

    ' display error message in page
    ' cannot use ID as label is not declared directly
    ' in the page and so would get a compiler error
    ' use Page-level reference to control saved in Page_load
    oLabel.Text = oErr.Message & "<p />"

  End Try

End Function

<%-------------------------------------------------------------%>

Sub DoItemEdit(s As Object, e As DataGridCommandEventArgs)

  ' get a reference to the DataGrid
  ' cannot use ID as grid is not declared directly
  ' in page so would get a compiler error
  ' use Page-level reference to control saved in Page_load
  ' set the EditItemIndex of the grid to this item's index
  oGrid.EditItemIndex = e.Item.ItemIndex

  ' bind grid to display row in new mode
  oGrid.DataSource = GetCustomers()
  oGrid.DataBind()

End Sub

<%-------------------------------------------------------------%>

Sub DoItemUpdate(s As Object, e As DataGridCommandEventArgs)

  ' get a reference to the DataGrid
  ' cannot use ID as grid is not declared directly
  ' in page so would get a compiler error
  ' alternative approach if reference not saved in Page_Load
  Dim oDGR As DataGrid = CType(Page.FindControl("dgr1"), DataGrid)

  ' get a reference to the text boxes in edit row
  Dim oCompanyName As TextBox = CType(e.Item.Cells(2).Controls(0), TextBox)
  Dim oCity As TextBox = CType(e.Item.Cells(3).Controls(0), TextBox)
  Dim oCountry As TextBox = CType(e.Item.Cells(4).Controls(0), TextBox)
  Dim oPhone As TextBox = CType(e.Item.Cells(5).Controls(0), TextBox)

  ' create a suitable SQL statement and execute it
  Dim sSQL As String
  sSQL = "UPDATE Customers SET CompanyName='" & oCompanyName.Text & "', " _
       & "City='" & oCity.Text & "', " _
       & "Country='" & oCountry.Text & "', " _
       & "Phone='" & oPhone.Text & "' " _
       & "WHERE CustomerID='" & oGrid.DataKeys(e.Item.ItemIndex) & "'"
  ExecuteSQLStatement(sSQL)

  ' set EditItemIndex of grid to -1 to switch out of Edit mode
  oGrid.EditItemIndex = -1

  ' bind grid to display row in new mode
  oGrid.DataSource = GetCustomers()
  oGrid.DataBind()

End Sub

<%-------------------------------------------------------------%>

Sub DoItemCancel(s As Object, e As DataGridCommandEventArgs)

  ' set EditItemIndex of grid to -1 to switch out of Edit mode
  ' use reference to DataGrid saved in Page_Load
  oGrid.EditItemIndex = -1

  ' bind grid to display row in new mode
  oGrid.DataSource = GetCustomers()
  oGrid.DataBind()

End Sub

<%-------------------------------------------------------------%>

Sub ExecuteSQLStatement(sSQL)

  ' execute SQL statement against the original data source
  Dim sConnect As String _
    = ConfigurationSettings.AppSettings("NorthwindOleDbConnectString")
  Dim oConnect As New OleDbConnection(sConnect)

  Try

    oConnect.Open()
    Dim oCommand As New OleDbCommand(sSQL, oConnect)
    If oCommand.ExecuteNonQuery() <> 1 Then
      ' use reference to Label saved in Page_Load
      oLabel.Text &= "ERROR: Could not update the selected row<p />"
    End If
    oConnect.Close()

  Catch oErr As Exception

    ' be sure to close connection if error occurs
    If oConnect.State <> ConnectionState.Closed  Then
      oConnect.Close()
    End If

    ' display error message in page
    ' alternative way to reference Label using FindControl
    Dim oLbl As Label = CType(Page.FindControl("lblErr"), Label)
    oLbl.Text &= "ERROR: " & oErr.Message & "<p />"

  End Try

End Sub

</script>

<%-------------------------------------------------------------%>
<%-------------------------------------------------------------%>

<html>
<head>
<!-- #include file="../global/style.inc" -->
<title>Creating a DataGrid Control Dynamically</title>
</head>
<body>

<span class="heading">Creating a DataGrid Control Dynamically</span><hr />

<form id="frmMain" runat="server">

  <!-- DataGrid and Label will be dynamically inserted here -->

</form>

<span class="cite">
Remember to edit the connection strings in your web.config
file for the Northwind sample database.
</span>
<hr />
<!-- #include file="../global/viewsource.inc" -->
<!-- #include file="../global/footer.inc" -->

</body>
</html>

