<%@Page Language="VB" Debug="True" %>
<%@Import Namespace="System.Data" %>
<%@Import Namespace="System.Data.OleDb" %>

<%-------------------------------------------------------------%>

<script runat="server">

' page level variable to hold a DataSet
Dim oDS As DataSet

Sub Page_Load()

  If Not Page.IsPostback Then
    BindDataGrid()
  End If

End Sub

<%-------------------------------------------------------------%>

Sub BindDataGrid()

  ' try and get DataSet from current user's Session
  oDS = CType(Session("inx11vdgr"), DataSet)

  If oDS Is Nothing Then

    lblErr.Text &= "Loaded DataSet from the database<br />"

    ' declare SQL statements to fill three tables
    Dim sProducts As String _
      = "SELECT ProductID, ProductName, Products.SupplierID, " _
      & "Products.CategoryID, UnitPrice, Discontinued, " _
      & "Supplier=Suppliers.CompanyName, " _
      & "Category=Categories.CategoryName " _
      & "FROM Products JOIN Suppliers " _
      & "  ON Products.SupplierID=Suppliers.SupplierID " _
      & "  JOIN Categories " _
      & "  ON Products.CategoryID=Categories.CategoryID " _
      & "WHERE ProductID <= 10 ORDER BY ProductName"

    Dim sSuppliers As String _
      = "SELECT SupplierID, CompanyName FROM Suppliers " _
      & "ORDER BY CompanyName"

    Dim sCategories As String _
      = "SELECT CategoryID, CategoryName FROM Categories " _
      & "ORDER BY CategoryName"

    Dim sConnect As String _
      = ConfigurationSettings.AppSettings("NorthwindOleDbConnectString")

    Dim oConnect As New OleDbConnection(sConnect)

    Try

      ' create new DataSet and fill from database
      oDS = New DataSet()
      Dim oDA As New OleDbDataAdapter(sProducts, oConnect)
      oConnect.Open()
      oDA.Fill(oDS, "Products")
      oDA.SelectCommand.CommandText = sSuppliers
      oDA.Fill(oDS, "Suppliers")
      oDA.SelectCommand.CommandText = sCategories
      oDA.Fill(oDS, "Categories")
      oConnect.Close()

      ' save DataSet in current user's Session
      Session("inx11vdgr") = oDS

    Catch oErr As Exception

      ' be sure to close connection if error occurs
      If oConnect.State <> ConnectionState.Closed  Then
        oConnect.Close()
      End If

      ' display error message in page
      lblErr.Text &= oErr.Message & "<br />"

    End Try

  End If

  ' bind DataGrid to Products table
  dgr1.DataSource = oDS
  dgr1.DataMember = "Products"
  dgr1.DataBind()

End Sub

<%-------------------------------------------------------------%>

Sub BindRowData(sender As Object, e As DataGridItemEventArgs)

  ' see what type of row (header, footer, item, etc.) caused the event
  Dim oType As ListItemType = CType(e.Item.ItemType, ListItemType)

  ' only process it if it's the row in Edit mode
  If oType = ListItemType.EditItem Then

    ' get a reference to DropDownList control in the edit row
    Dim oSupplier As DropDownList = CType(e.Item.FindControl("lstSupplier"), DropDownList)

    ' bind it to the Suppliers table
    oSupplier.DataSource = oDS
    oSupplier.DataMember = "Suppliers"
    oSupplier.DataTextField = "CompanyName"
    oSupplier.DataValueField = "SupplierID"
    oSupplier.DataBind()

    ' set the current selection to the row value
    oSupplier.SelectedValue = e.Item.DataItem("SupplierID").ToString()

    'repeat for Categories RadioButtonList
    Dim oCategory As RadioButtonList = CType(e.Item.FindControl("lstCategory"), RadioButtonList)

    ' bind it to the Categories table
    oCategory.DataSource = oDS
    oCategory.DataMember = "Categories"
    oCategory.DataTextField = "CategoryName"
    oCategory.DataValueField = "CategoryID"
    oCategory.DataBind()
    oCategory.SelectedValue = e.Item.DataItem("CategoryID").ToString()

  End If

End Sub

<%-------------------------------------------------------------%>

Sub DoItemEdit(sender As Object, e As DataGridCommandEventArgs)

  ' set the EditItemIndex of the grid to this item's index
  dgr1.EditItemIndex = e.Item.ItemIndex

  ' bind grid to display newly-loaded data
  BindDataGrid()

  ' display the validation error hints
  helptext.Visible = True

End Sub

<%-------------------------------------------------------------%>

Sub DoItemUpdate(sender As Object, e As DataGridCommandEventArgs)

  If Page.IsValid Then

    ' remove existing DataSet from current user's Session
    Session("inx11vdgr") = Nothing

    ' get a reference to controls in the edit row
    Dim oSupplier As DropDownList = CType(e.Item.FindControl("lstSupplier"), DropDownList)
    Dim oCategory As RadioButtonList = CType(e.Item.FindControl("lstCategory"), RadioButtonList)
    Dim oPrice As TextBox = CType(e.Item.FindControl("txtPrice"), TextBox)
    Dim oDisc As CheckBox = CType(e.Item.FindControl("chkDiscontinued"), CheckBox)

    ' create a suitable SQL statement and execute it
    Dim sSQL As String
    sSQL = "UPDATE Products SET SupplierID=" & oSupplier.SelectedValue & ", " _
         & "CategoryID=" & oCategory.SelectedValue & ", " _
         & "UnitPrice=" & oPrice.Text & ", " _
         & "Discontinued=" & CType(oDisc.Checked, Int16) & " " _
         & "WHERE ProductID=" & dgr1.DataKeys(e.Item.ItemIndex)
    ExecuteSQLStatement(sSQL)

    ' set EditItemIndex of grid to -1 to switch out of Edit mode
    dgr1.EditItemIndex = -1

    ' bind grid to display row in new mode
    BindDataGrid()

    ' hide the validation error hints
    helptext.Visible = False

  End If

End Sub

<%-------------------------------------------------------------%>

Sub DoItemCancel(sender As Object, e As DataGridCommandEventArgs)

  ' set EditItemIndex of grid to -1 to switch out of Edit mode
  dgr1.EditItemIndex = -1

  ' bind grid to display row in new mode
  BindDataGrid()

  ' hide the validation error hints
  helptext.Visible = False

End Sub

<%-------------------------------------------------------------%>

Sub ExecuteSQLStatement(sSQL)

  ' execute SQL statement against the original data source
  Dim sConnect As String _
    = ConfigurationSettings.AppSettings("NorthwindOleDbConnectString")
  Dim oConnect As New OleDbConnection(sConnect)

  Try

    oConnect.Open()
    Dim oCommand As New OleDbCommand(sSQL, oConnect)
    If oCommand.ExecuteNonQuery() <> 1 Then
      lblErr.Text &= "ERROR: Could not update the selected row<br />"
    End If
    oConnect.Close()

  Catch oErr As Exception

    ' be sure to close connection if error occurs
    If oConnect.State <> ConnectionState.Closed  Then
      oConnect.Close()
    End If

    ' display error message in page
    lblErr.Text &= "ERROR: " & oErr.Message & "<br />"

  End Try

End Sub

<%-------------------------------------------------------------%>

Sub KillSession(sender As Object, e As EventArgs)

  ' remove existing DataSet from current user's Session
  Session("inx11vdgr") = Nothing

  ' bind grid to display newly-loaded data
  BindDataGrid()

End Sub

<%-------------------------------------------------------------%>

Sub ServerValidatePrice(sender As Object, _
                        e As ServerValidateEventArgs)

  Dim bValid As Boolean = False
  Dim sValue As String = e.Value
  If  sValue.Length > 0 Then
    Dim sLast As String = sValue.Substring(sValue.Length - 1)
    If "059".IndexOf(sLast) <> -1 Then
      bValid = True
    End If
  End If
  e.IsValid = bValid

End Sub

</script>

<%-------------------------------------------------------------%>
<%-------------------------------------------------------------%>

<html>
<head>
<!-- #include file="../global/style.inc" -->
<title>Using List and Validation Controls in a DataGrid</title>

<script language="JavaScript">
<!--
// client-side validation function for CustomValidator
function ClientValidatePrice(source, args) {
  var bValid = false;
  var sValue = args.Value.toString();
  var sLast = sValue.substring(sValue.length - 1, sValue.length)
  if ('059'.indexOf(sLast) != -1)
    bValid = true;
  args.IsValid = bValid;
  return;
}
//-->
</script>

</head>
<body>

<span class="heading">Using List and Validation Controls in a DataGrid</span><hr />

<form runat="server">

  <asp:DataGrid id="dgr1" runat="server"
       Font-Size="10" Font-Name="Tahoma,Arial,Helvetica,sans-serif"
       BorderStyle="None" BorderWidth="1px" BorderColor="#deba84"
       BackColor="#DEBA84" CellPadding="5" CellSpacing="1"
       DataKeyField="ProductID"
       OnEditCommand="DoItemEdit"
       OnUpdateCommand="DoItemUpdate"
       OnCancelCommand="DoItemCancel"
       OnItemDataBound="BindRowData"
       AutoGenerateColumns="False">
    <HeaderStyle Font-Bold="True" ForeColor="#ffffff" BackColor="#b50055" />
    <ItemStyle BackColor="#FFF7E7" VerticalAlign="Top" />
    <AlternatingItemStyle BackColor="#ffffc0" />

    <Columns>

      <asp:BoundColumn DataField="ProductID" HeaderText="ID"
           HeaderStyle-HorizontalAlign="Center" ReadOnly="True" />

      <asp:BoundColumn DataField="ProductName" HeaderText="Product"
           HeaderStyle-HorizontalAlign="Center" ReadOnly="True" />

      <asp:TemplateColumn HeaderText="Supplier"
           HeaderStyle-HorizontalAlign="Center" >
        <ItemTemplate>
          <%# Container.DataItem("Supplier") %>
        </ItemTemplate>
        <EditItemTemplate>
          <asp:DropDownList id="lstSupplier" runat="server" />
          <asp:CompareValidator id="valSupplier" runat="server"
               ControlToValidate="lstSupplier"
               ControlToCompare="lstCategory"
               Operator="NotEqual"
               ErrorMessage="This combination of supplier and category is not valid"
               Display="Dynamic" Text="*" />
        </EditItemTemplate>
      </asp:TemplateColumn>

      <asp:TemplateColumn HeaderText="Category">
        <ItemTemplate>
          <%# Container.DataItem("Category") %>
        </ItemTemplate>
        <EditItemTemplate>
          <asp:RadioButtonList id="lstCategory" runat="server" />
          <asp:CompareValidator id="valCategory" runat="server"
               ControlToValidate="lstCategory"
               Operator="NotEqual"
               ValueToCompare="5"
               ErrorMessage="This category is no longer available"
               Display="Dynamic" Text="*" />
        </EditItemTemplate>
      </asp:TemplateColumn>

      <asp:TemplateColumn HeaderText="Price"
           HeaderStyle-HorizontalAlign="Center"
           ItemStyle-HorizontalAlign="Right">
        <ItemTemplate>
          <%# DataBinder.Eval(Container.DataItem, "UnitPrice", "${0:F2}") %>
        </ItemTemplate>
        <EditItemTemplate>
          $<asp:TextBox Columns="3" id="txtPrice" runat="server"
                Text='<%# DataBinder.Eval(Container.DataItem, "UnitPrice", "{0:F2}") %>' />
          <asp:RequiredFieldValidator id="valPrice1" runat="server"
               ControlToValidate="txtPrice"
               ErrorMessage="You must enter a price"
               Display="Dynamic" Text="*" />
          <asp:RangeValidator id="valPrice2" runat="server"
               ControlToValidate="txtPrice"
               MaximumValue="999.99"
               MinimumValue="0.99"
               ErrorMessage="The price must be between $0.99 and $999.99"
               Display="Dynamic" Text="*" />
          <asp:CustomValidator id="valPrice3" runat="server"
               ControlToValidate="txtPrice"
               ClientValidationFunction="ClientValidatePrice"
               OnServerValidate="ServerValidatePrice"
               ErrorMessage="The price must end with 0, 5 or 9"
               Display="Dynamic" Text="*" />
        </EditItemTemplate>
      </asp:TemplateColumn>

      <asp:TemplateColumn HeaderText="Discontinued"
           ItemStyle-HorizontalAlign="Center">
        <ItemTemplate>
          <asp:Image Width="12" Height="12" Border="0" runat="server"
            AlternateText='<%# DataBinder.Eval(Container.DataItem, "Discontinued", "{0:G}") %>'
            ImageUrl='<%# DataBinder.Eval(Container.DataItem, "Discontinued", "~/images/{0:G}.gif") %>' />
        </ItemTemplate>
        <EditItemTemplate>
          <asp:CheckBox id="chkDiscontinued" runat="server"
               Checked='<%# Container.DataItem("Discontinued") %>' />
        </EditItemTemplate>
      </asp:TemplateColumn>

      <asp:EditCommandColumn EditText="Edit"
           CancelText="Cancel" UpdateText="Update" />

    </Columns>
  </asp:DataGrid>

<div id="helptext" class="cite" Visible="False" runat="server">
To force a validation error:<br />
* Select category 'Grains/Cereals'<br />
* Select supplier 'Grandma Kellys Homestead' and category 'Confections'<br />
* Leave the 'price' textbox empty or enter a non-numeric value<br />
* Enter a price that does not end with 0, 5 or 9
<p />
Remember to edit the connection strings in your web.config
file for the Northwind sample database.
</div>

<p />

<asp:ValidationSummary id="valSummary" runat="server"
     DisplayMode="BulletList"
     HeaderText="<b>The following errors were detected:</b>" />

<asp:Button Text="Kill Session" id="btnKill" OnClick="KillSession" runat="server" />
&nbsp;
<asp:Label id="lblErr" EnableViewState="False" runat="server" />

</form>

<hr />
<!-- #include file="../global/viewsource.inc" -->
<!-- #include file="../global/footer.inc" -->

</body>
</html>

